function getNumber (input) {
    let i = 0
    while (i < input.length && '-.0123456789'.includes(input[i])) {
       i++
    }
    if (i === 0) return [[], input]
    let token = {
        type: 'number',
        value: Number(input.slice(0, i))
    }
    return [[token], input.slice(i)]
};

getNumber("123");
getNumber("1234 6789");
getNumber(" 12");

function getQuotedString (input) {
    if (input.length === 0) return [[], input]
    let i = 0
    let q = input[i++]
    if (!"'\"".includes(q)) return [[], input]
    while (i < input.length) {
       if (input[i] === q) break
       i++
    }
    let token = {
        type: 'string',
        value: input.slice(1, i)
    }
    return [[token], input.slice(i+1)]
};
getQuotedString('hello');
getQuotedString('"hello"');
getQuotedString("'hello world'");

function getWhitespace (input) {
    let i = 0
    while (i < input.length && ' \t\n\r'.includes(input[i])) {
       i++
    }
    if (i === 0) return [[], input]
    let token = {
        type: 'whitespace',
        value: input.slice(0, i)
    }
    return [[token], input.slice(i)]
};
getWhitespace('hello  ');
getWhitespace(' hello');
getWhitespace('\thello');
getWhitespace('   \n   ');

function getRawString (input) {
    if (input.length === 0) return [[], input]
    let i = 0
    while (i < input.length && !' \t\n\r'.includes(input[i])) {
       i++
    }
    if (i===0) return [[], input]
    let token = {
        type: 'word',
        value: input.slice(0, i)
    }
    return [[token], input.slice(i)]
};

function getTokens (input) {
    let parsers = [getNumber, getQuotedString, getRawString, getWhitespace]
    let result = []
    let t
    let lastsize = Infinity
    while (input.length > 0 && input.length < lastsize) {
        lastsize = input.length
        for (parser of parsers) {
            [t, input] = parser(input)
            result = [...result, ...t]
        }
    }    
    return [result, input]
}
;

getTokens(`echo 'I say "hello"!' 1 2`);
